#!/bin/bash
#
# dir-file-add-add.sh
#
# Test whether a dual-add (where one is a directory and the other a
# file) merge conflict introduces subtree merge info.

set -e

SERVER_DIR=server
CLIENT_DIR=client

if [ $# != 1 ]; then
    echo "usage: $0 <path to svn>"
    exit
fi

SVN=$1
SVNADMIN="$(dirname $SVN)/svnadmin"

SERVER_URL="file:///$PWD/$SERVER_DIR"

# Create the repo and enter it.
$SVNADMIN create $SERVER_DIR
$SVN checkout $SERVER_URL $CLIENT_DIR
cd $CLIENT_DIR

# Create the trunk/branches structure
mkdir trunk
mkdir branches
$SVN add trunk branches
$SVN commit -m 'Create initial structure'

# Make a branch from the trunk.
$SVN copy ^/trunk ^/branches/branch -m 'Create branch from trunk'

# Commit a subdir with a file in it in the trunk and commit it.
# NB: If testing version 1.6 or earlier, remove "--ignore-ancestry".
$SVN switch --ignore-ancestry ^/trunk ||
    $SVN switch ^/trunk

# Create a subdir with a file in it on trunk and commit it.
mkdir dir
touch dir/file.txt
$SVN add dir
$SVN commit -m 'Added dir/file.txt on trunk'

# Commit a file named 'dir' on branch.
$SVN switch ^/branches/branch
touch dir
$SVN add dir
$SVN commit -m 'Added file named "dir" on branch'

# Merge the trunk.
$SVN update
$SVN merge --non-interactive ^/trunk || true

# Resolve the tree conflict.
$SVN resolve --accept=working dir ||
    $SVN resolved dir

# See if the svn:mergeinfo properties are what we expect.
$SVN propget svn:mergeinfo -R . >actual.out
echo '. - /trunk:2-4' >expected.out

$SVN --version | head -1

if diff -q actual.out expected.out; then
    echo Success
else
    echo 'Test failed!'
    echo 'Expected output:'
    cat expected.out
    echo 'Actual output:'
    cat actual.out
    exit 1
fi
